/*********************************************************************
 *
 *                Microchip USB C18 Firmware - Mouse Demo
 *
 *********************************************************************
 * FileName:        user_mouse.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 * Steffen Schtte      02/05/08    Added support for ADNS 2610 mouse sensor
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include <delays.h>
#include "system\typedefs.h"

#include "system\usb\usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user\user_mouse.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
//byte old_sw2,old_sw3;

//BOOL emulate_mode;
rom signed char dir_table[]={-4,-4,-4, 0, 4, 4, 4, 0};
byte movement_length;
//byte vector = 0;
char buffer[3];

//Movement of the sensor
byte x;
byte y;


/** P R I V A T E  P R O T O T Y P E S ***************************************/
void BlinkUSBStatus(void);
BOOL Switch2IsPressed(void);
BOOL Switch3IsPressed(void);

void Emulate_Mouse(void);

void maus_sens_writeByte(byte data);

byte maus_sens_readByte(void);

void maus_sens_write(byte adr, byte data);

byte maus_sens_read(byte adr);

void maus_sens_init(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
    mInitAllLEDs();
    mInitAllSwitches();

    buffer[0]=0;
    buffer[1]=0;
    buffer[2]=0;
    buffer[3]=0;
    
     
	T3CON = 0b00010001; //timer3 on, 1:2 prescaler
	PIE2bits.TMR3IE = 1;
	INTCONbits.PEIE = 1;
	
	ADCON1 = 0x0e; //Default to digital
	
	TRISC = 0x00; //All output SDO is RC7
	TRISBbits.TRISB0 = 0; //SDI
	TRISBbits.TRISB1 = 0; //SCK
	
	TRISBbits.TRISB6 = 0; //Oszi trigger
	
	maus_sens_init();
    
}//end UserInit



/*!
 * Uebertraegt ein Byte an den Sensor
 * @param data das Byte
 */
void maus_sens_writeByte(byte data){
	byte i;
    TRISBbits.TRISB0 = 0; 		// SDA auf Output
	
	for (i=0; i<8; i++){
		PORTBbits.RB1 = 0;		// SCK auf Low, vorbereiten
		
		//Daten rausschreiben
		if(data & 0x80)
		{
			PORTBbits.RB0 = 1;			
		}
		else
		{
			PORTBbits.RB0 = 0;
		}
		data = data <<1;		// naechstes Bit vorbereiten
		
		Delay10TCYx(1);
		
		PORTBbits.RB1 = 1;		// SCK =1 Sensor uebernimmt auf steigender Flanke
		
		Delay10TCYx(1);
	}
	
    TRISBbits.TRISB0 = 1; 		// SDA auf Input
}

/*!
 * Liest ein Byte vom Sensor
 * @return das Byte
 */
byte maus_sens_readByte(void){
	int i;
	byte data=0;

    TRISBbits.TRISB0 = 1; 		// SDA auf Input

	for (i=0; i<8; i++){
		PORTBbits.RB1 = 0;		// SCK =0 Sensor bereitet Daten auf fallender Flanke vor !
		data=data<<1;					// Platz schaffen

		Delay10TCYx(1);
		PORTBbits.RB1 = 1;		// SCK =1 Daten lesen  auf steigender Flanke
		
		data |= PORTBbits.RB0;			//Daten lesen
		
		Delay10TCYx(1);
	}

	return data;
}

/*!
 * Uebertraegt ein write-Kommando an den Sensor
 * @param adr Adresse
 * @param data Datum
 */
void maus_sens_write(byte adr, byte data){

	maus_sens_writeByte(adr|=0x80);  //rl MSB muss 1 sein Datenblatt S.12 Write Operation
	maus_sens_writeByte(data);
	Delay100TCYx(15);	// mindestens 100 Mikrosekunden Pause!!!
}

/*!
 * Schickt ein Lesekommando an den Sensor
 * und liest ein Byte zurueck
 * @param adr die Adresse
 * @return das Datum
 */
byte maus_sens_read(byte adr){
	maus_sens_writeByte(adr);
	return maus_sens_readByte();
}

/*! 
 * Initialisiere Maussensor
 */ 
void maus_sens_init(void){
	
	maus_sens_write(0x00,0x80);	//Reset sensor
	maus_sens_write(0x00,0x01);	//Always on
}


/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   

    INTCONbits.GIE = 1;
	Delay10TCYx(1); //time for firing the interrupt (sensor read)
	INTCONbits.GIE = 0;
    BlinkUSBStatus();
    // User Application USB tasks
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;

     	
    Emulate_Mouse();
     
}//end ProcessIO

void Emulate_Mouse(void)
{

        if(movement_length > 20)
        {
			buffer[0] = 0;
			if(!sw2)
				buffer[0] = 2;
			if(!sw3)
				buffer[0]++;

            buffer[1] = x;
            buffer[2] = 0x00-y;//Invert direction
            //vector++;
            movement_length = 0;
        }//end if(movement_length > 20)



    if(!mHIDTxIsBusy())
    {
	    if(movement_length > 10)
	    {
	        HIDTxReport(buffer,3);
	    }
        movement_length++;
    }//end if(mHIDIsPutReportReady())
}//end Emulate_Mouse

/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            mLED_2 = mLED_1;        // Both blink at the same time
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(usb_device_state == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_Toggle();
                mLED_2 = !mLED_1;       // Alternate blink                
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

/*BOOL Switch2IsPressed(void)
{
    if(sw2 != old_sw2)
    {
        old_sw2 = sw2;                  // Save new value
        if(sw2 == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end Switch2IsPressed

BOOL Switch3IsPressed(void)
{
    if(sw3 != old_sw3)
    {
        old_sw3 = sw3;                  // Save new value
        if(sw3 == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end Switch3IsPressed
*/

//User interrupt service routine
void user_interrupt(void)
{


	PORTBbits.RB4 = 1;
	y = maus_sens_read(0x02);
	x = maus_sens_read(0x03);
	PORTBbits.RB4 = 0;

			
	mLED_3_Toggle();	

	if(PIR2bits.TMR3IF)
	{	
		TMR3H += 22; //correction factor for ~100Hz interrupt rate
		PIR2bits.TMR3IF = 0;
	}
}

/** EOF user_mouse.c *********************************************************/
